
#include "Atomic/CorpSCADANetworks/CorpSCADANetworksADVISE.h"
#include <cstdlib>
#include <iostream>
#include <cmath>
#include <array>


CorpSCADANetworksADVISE::CorpSCADANetworksADVISE() {
  DoNothingAD.setParent(this);
  SCADALAN_InstallMalwareOnNetworkAD.setParent(this);
  CorpSCADAFW_DefeatFirewallAD.setParent(this);
  DoNothingOutcome1.setParent(this);
  SCADALAN_InstallMalwareOnNetworkFailure.setParent(this);
  SCADALAN_InstallMalwareOnNetworkSuccess.setParent(this);
  CorpSCADAFW_DefeatFirewallFailure.setParent(this);
  CorpSCADAFW_DefeatFirewallSuccess.setParent(this);
  SCADALAN_InstallMalwareOnNetworkGroup.initialize(2, "SCADALAN_InstallMalwareOnNetworkGroup");
  SCADALAN_InstallMalwareOnNetworkGroup.appendGroup((BaseGroupClass*) &SCADALAN_InstallMalwareOnNetworkFailure);
  SCADALAN_InstallMalwareOnNetworkGroup.appendGroup((BaseGroupClass*) &SCADALAN_InstallMalwareOnNetworkSuccess);

  CorpSCADAFW_DefeatFirewallGroup.initialize(2, "CorpSCADAFW_DefeatFirewallGroup");
  CorpSCADAFW_DefeatFirewallGroup.appendGroup((BaseGroupClass*) &CorpSCADAFW_DefeatFirewallFailure);
  CorpSCADAFW_DefeatFirewallGroup.appendGroup((BaseGroupClass*) &CorpSCADAFW_DefeatFirewallSuccess);

  AdversaryDecisionGroup.initialize(3, "AdversaryDecisionGroup");
  AdversaryDecisionGroup.appendGroup((BaseGroupClass*) &DoNothingAD);
  AdversaryDecisionGroup.appendGroup((BaseGroupClass*) &SCADALAN_InstallMalwareOnNetworkAD);
  AdversaryDecisionGroup.appendGroup((BaseGroupClass*) &CorpSCADAFW_DefeatFirewallAD);

  BaseADVISEAction* InitialActions[8] = {
    &DoNothingAD, // 0
    &SCADALAN_InstallMalwareOnNetworkAD, // 1
    &CorpSCADAFW_DefeatFirewallAD, // 2
    &DoNothingOutcome1, // 3
    &SCADALAN_InstallMalwareOnNetworkFailure, // 4
    &SCADALAN_InstallMalwareOnNetworkSuccess, // 5
    &CorpSCADAFW_DefeatFirewallFailure, // 6
    &CorpSCADAFW_DefeatFirewallSuccess // 7
  };

  Step* InitialSteps[5] = {
      &DoNothingOutcome1,
      &SCADALAN_InstallMalwareOnNetworkFailure,
      &SCADALAN_InstallMalwareOnNetworkSuccess,
      &CorpSCADAFW_DefeatFirewallFailure,
      &CorpSCADAFW_DefeatFirewallSuccess
  };

  BaseGroupClass* InitialGroups[4] = {
    (BaseGroupClass*) &DoNothingOutcome1, // 0
    (BaseGroupClass*) &SCADALAN_InstallMalwareOnNetworkGroup, // 1
    (BaseGroupClass*) &CorpSCADAFW_DefeatFirewallGroup, // 2
    (BaseGroupClass*) &AdversaryDecisionGroup
  };

  CorporateLAN_NetworkAccess = new Access("CorporateLAN_NetworkAccess", 1);
  SCADALAN_NetworkAccess = new Access("SCADALAN_NetworkAccess", 0);
  BruteForceSkill = new Skill("BruteForceSkill", 400);
  SCADALAN_MalwareInstalled = new SystemSV<bool>("SCADALAN_MalwareInstalled", false);
  SCADALAN_isIDSOperational = new SystemSV<bool>("SCADALAN_isIDSOperational", true);
  MalwareonSCADANet = new Goal("MalwareonSCADANet", 0);
  MalwareonSCADANet->setPayoff(500);
  MakeDecision = new BeginAdversaryDecision("MakeDecision", 0);
  DoNothingWeight = new StepWeight("DoNothingWeight", 0);
  DoNothingChosen = new StepChosen("DoNothingChosen", 0);
  SCADALAN_InstallMalwareOnNetworkWeight = new StepWeight("SCADALAN_InstallMalwareOnNetworkWeight", 0);
  SCADALAN_InstallMalwareOnNetworkChosen = new StepChosen("SCADALAN_InstallMalwareOnNetworkChosen", 0);
  CorpSCADAFW_DefeatFirewallWeight = new StepWeight("CorpSCADAFW_DefeatFirewallWeight", 0);
  CorpSCADAFW_DefeatFirewallChosen = new StepChosen("CorpSCADAFW_DefeatFirewallChosen", 0);

  Goal* InitialGoals[1] = {
      MalwareonSCADANet
  };

  BaseStateVariableClass* InitialSVs[13] = {
    CorporateLAN_NetworkAccess, // 0
    SCADALAN_NetworkAccess, // 1
    BruteForceSkill, // 2
    SCADALAN_MalwareInstalled, // 3
    SCADALAN_isIDSOperational, // 4
    MalwareonSCADANet, // 5
    DoNothingChosen, // 6
    SCADALAN_InstallMalwareOnNetworkChosen, // 7
    CorpSCADAFW_DefeatFirewallChosen, // 8
    DoNothingWeight, // 9
    SCADALAN_InstallMalwareOnNetworkWeight, // 10
    CorpSCADAFW_DefeatFirewallWeight, // 11
    MakeDecision // 12
};

  int outCounts[3] = { 1, 2, 2 };

  commonInit("CorpSCADANetworks", 13, InitialSVs, 8, InitialActions, 4, InitialGroups, 3, outCounts, InitialSteps, 1, InitialGoals);

  advCostPref = 0.2;
  advDetectPref = 0.05;
  advPayPref = 0.75;

  planningHorizon = 2;
  costDiscount = 1.0;
  detectionDiscount = 1.0;
  payoffDiscount = 1.0;

  assignSVsToAttackSteps();

  int AffectArcs[46][2] = {
    {12,0}, {6,0}, {9,0}, {12,1}, {7,1}, {10,1}, {12,2}, {8,2}, 
    {11,2}, {12,3}, {6,3}, {9,3}, {10,3}, {11,3}, {12,4}, {7,4}, 
    {9,4}, {10,4}, {11,4}, {3,4}, {5,4}, {1,4}, {4,4}, {12,5}, 
    {7,5}, {9,5}, {10,5}, {11,5}, {3,5}, {5,5}, {1,5}, {4,5}, 
    {12,6}, {8,6}, {9,6}, {10,6}, {11,6}, {0,6}, {1,6}, {12,7}, 
    {8,7}, {9,7}, {10,7}, {11,7}, {0,7}, {1,7}
};

  for(int n = 0; n < 46; ++n)
    AddAffectArc(InitialSVs[AffectArcs[n][0]], InitialActions[AffectArcs[n][1]]);
  int EnableArcs[16][2] = {
    {12,0}, {12,1}, {12,2}, {6,3}, {7,4}, {3,4}, {5,4}, {7,5}, 
    {3,5}, {5,5}, {8,6}, {0,6}, {1,6}, {8,7}, {0,7}, {1,7}
    
};

  for(int n = 0; n < 16; ++n)
    AddEnableArc(InitialSVs[EnableArcs[n][0]], InitialActions[EnableArcs[n][1]]);

  for(int x = 0; x < 8; ++x) {
    InitialActions[x]->LinkVariables();
  }

  customInitialization();

  recalculateAdversaryDecisionWeights();
}

CorpSCADANetworksADVISE::~CorpSCADANetworksADVISE() {
}

void CorpSCADANetworksADVISE::assignSVsToAttackSteps() {
  DoNothingOutcome1.DoNothingChosen = DoNothingChosen;
  DoNothingOutcome1.DoNothingWeight = DoNothingWeight;
  DoNothingOutcome1.SCADALAN_InstallMalwareOnNetworkWeight = SCADALAN_InstallMalwareOnNetworkWeight;
  DoNothingOutcome1.CorpSCADAFW_DefeatFirewallWeight = CorpSCADAFW_DefeatFirewallWeight;
  DoNothingOutcome1.MakeDecision = MakeDecision;
  DoNothingOutcome1.setSVs(DoNothingChosen, DoNothingWeight);
  SCADALAN_InstallMalwareOnNetworkFailure.SCADALAN_MalwareInstalled = SCADALAN_MalwareInstalled;
  SCADALAN_InstallMalwareOnNetworkFailure.MalwareonSCADANet = MalwareonSCADANet;
  SCADALAN_InstallMalwareOnNetworkFailure.SCADALAN_NetworkAccess = SCADALAN_NetworkAccess;
  SCADALAN_InstallMalwareOnNetworkFailure.SCADALAN_isIDSOperational = SCADALAN_isIDSOperational;
  SCADALAN_InstallMalwareOnNetworkFailure.SCADALAN_InstallMalwareOnNetworkChosen = SCADALAN_InstallMalwareOnNetworkChosen;
  SCADALAN_InstallMalwareOnNetworkFailure.DoNothingWeight = DoNothingWeight;
  SCADALAN_InstallMalwareOnNetworkFailure.SCADALAN_InstallMalwareOnNetworkWeight = SCADALAN_InstallMalwareOnNetworkWeight;
  SCADALAN_InstallMalwareOnNetworkFailure.CorpSCADAFW_DefeatFirewallWeight = CorpSCADAFW_DefeatFirewallWeight;
  SCADALAN_InstallMalwareOnNetworkFailure.MakeDecision = MakeDecision;
  SCADALAN_InstallMalwareOnNetworkFailure.setSVs(SCADALAN_InstallMalwareOnNetworkChosen, SCADALAN_InstallMalwareOnNetworkWeight);
  SCADALAN_InstallMalwareOnNetworkSuccess.SCADALAN_MalwareInstalled = SCADALAN_MalwareInstalled;
  SCADALAN_InstallMalwareOnNetworkSuccess.MalwareonSCADANet = MalwareonSCADANet;
  SCADALAN_InstallMalwareOnNetworkSuccess.SCADALAN_NetworkAccess = SCADALAN_NetworkAccess;
  SCADALAN_InstallMalwareOnNetworkSuccess.SCADALAN_isIDSOperational = SCADALAN_isIDSOperational;
  SCADALAN_InstallMalwareOnNetworkSuccess.SCADALAN_InstallMalwareOnNetworkChosen = SCADALAN_InstallMalwareOnNetworkChosen;
  SCADALAN_InstallMalwareOnNetworkSuccess.DoNothingWeight = DoNothingWeight;
  SCADALAN_InstallMalwareOnNetworkSuccess.SCADALAN_InstallMalwareOnNetworkWeight = SCADALAN_InstallMalwareOnNetworkWeight;
  SCADALAN_InstallMalwareOnNetworkSuccess.CorpSCADAFW_DefeatFirewallWeight = CorpSCADAFW_DefeatFirewallWeight;
  SCADALAN_InstallMalwareOnNetworkSuccess.MakeDecision = MakeDecision;
  SCADALAN_InstallMalwareOnNetworkSuccess.setSVs(SCADALAN_InstallMalwareOnNetworkChosen, SCADALAN_InstallMalwareOnNetworkWeight);
  CorpSCADAFW_DefeatFirewallFailure.CorporateLAN_NetworkAccess = CorporateLAN_NetworkAccess;
  CorpSCADAFW_DefeatFirewallFailure.SCADALAN_NetworkAccess = SCADALAN_NetworkAccess;
  CorpSCADAFW_DefeatFirewallFailure.BruteForceSkill = BruteForceSkill;
  CorpSCADAFW_DefeatFirewallFailure.CorpSCADAFW_DefeatFirewallChosen = CorpSCADAFW_DefeatFirewallChosen;
  CorpSCADAFW_DefeatFirewallFailure.DoNothingWeight = DoNothingWeight;
  CorpSCADAFW_DefeatFirewallFailure.SCADALAN_InstallMalwareOnNetworkWeight = SCADALAN_InstallMalwareOnNetworkWeight;
  CorpSCADAFW_DefeatFirewallFailure.CorpSCADAFW_DefeatFirewallWeight = CorpSCADAFW_DefeatFirewallWeight;
  CorpSCADAFW_DefeatFirewallFailure.MakeDecision = MakeDecision;
  CorpSCADAFW_DefeatFirewallFailure.setSVs(CorpSCADAFW_DefeatFirewallChosen, CorpSCADAFW_DefeatFirewallWeight);
  CorpSCADAFW_DefeatFirewallSuccess.CorporateLAN_NetworkAccess = CorporateLAN_NetworkAccess;
  CorpSCADAFW_DefeatFirewallSuccess.SCADALAN_NetworkAccess = SCADALAN_NetworkAccess;
  CorpSCADAFW_DefeatFirewallSuccess.BruteForceSkill = BruteForceSkill;
  CorpSCADAFW_DefeatFirewallSuccess.CorpSCADAFW_DefeatFirewallChosen = CorpSCADAFW_DefeatFirewallChosen;
  CorpSCADAFW_DefeatFirewallSuccess.DoNothingWeight = DoNothingWeight;
  CorpSCADAFW_DefeatFirewallSuccess.SCADALAN_InstallMalwareOnNetworkWeight = SCADALAN_InstallMalwareOnNetworkWeight;
  CorpSCADAFW_DefeatFirewallSuccess.CorpSCADAFW_DefeatFirewallWeight = CorpSCADAFW_DefeatFirewallWeight;
  CorpSCADAFW_DefeatFirewallSuccess.MakeDecision = MakeDecision;
  CorpSCADAFW_DefeatFirewallSuccess.setSVs(CorpSCADAFW_DefeatFirewallChosen, CorpSCADAFW_DefeatFirewallWeight);

  DoNothingAD.MakeDecision = MakeDecision;
  DoNothingAD.DoNothingChosen = DoNothingChosen;
  DoNothingAD.DoNothingWeight = DoNothingWeight;
  SCADALAN_InstallMalwareOnNetworkAD.MakeDecision = MakeDecision;
  SCADALAN_InstallMalwareOnNetworkAD.SCADALAN_InstallMalwareOnNetworkChosen = SCADALAN_InstallMalwareOnNetworkChosen;
  SCADALAN_InstallMalwareOnNetworkAD.SCADALAN_InstallMalwareOnNetworkWeight = SCADALAN_InstallMalwareOnNetworkWeight;
  CorpSCADAFW_DefeatFirewallAD.MakeDecision = MakeDecision;
  CorpSCADAFW_DefeatFirewallAD.CorpSCADAFW_DefeatFirewallChosen = CorpSCADAFW_DefeatFirewallChosen;
  CorpSCADAFW_DefeatFirewallAD.CorpSCADAFW_DefeatFirewallWeight = CorpSCADAFW_DefeatFirewallWeight;
}

double CorpSCADANetworksADVISE::convertCostToUtility(double cost) {
  const double esquared = 7.389056099; // e^(2)
  if(cost < 0)
    return 1.0;
  else if(cost > 100)
    return 0.0;
  else {
    return (esquared - exp(cost/50))/(-1 + esquared);
  }
}

double CorpSCADANetworksADVISE::convertPayoffToUtility(double payoff) {
  const double etoten = 28.031624895; // e^(10/3)
  if(payoff < 0)
    return 0;
  else if(payoff > 1000)
    return 1;
  else {
    return (etoten - (etoten/exp(payoff/300)))/(-1 + etoten);
  }
}

double CorpSCADANetworksADVISE::convertDetectionToUtility(double detection) {
  const double esquared = 7.389056099; // e^(2)
  if(detection < 0)
    return 1;
  else if(detection > 1)
    return 0;
  else {
    return (1-(esquared/exp(2*detection)))/(1-esquared);
  }
}

void CorpSCADANetworksADVISE::customInitialization() {
}

/*****************************************************************/
/*                   Attack Step Definitions                     */
/*****************************************************************/

/*====================== DoNothingOutcome1Step ========================*/

CorpSCADANetworksADVISE::DoNothingOutcome1Step::DoNothingOutcome1Step() {
  TheDistributionParameters = new double[1];
  commonInit("DoNothingOutcome1Step", 0, Deterministic, RaceEnabled, 5, 0, false);
  setStepName("DoNothing");
}

CorpSCADANetworksADVISE::DoNothingOutcome1Step::~DoNothingOutcome1Step() {
  delete[] TheDistributionParameters;
}

void CorpSCADANetworksADVISE::DoNothingOutcome1Step::LinkVariables() {
  DoNothingChosen->Register(&DoNothingChosen_Mobius_Mark);
  DoNothingWeight->Register(&DoNothingWeight_Mobius_Mark);
  SCADALAN_InstallMalwareOnNetworkWeight->Register(&SCADALAN_InstallMalwareOnNetworkWeight_Mobius_Mark);
  CorpSCADAFW_DefeatFirewallWeight->Register(&CorpSCADAFW_DefeatFirewallWeight_Mobius_Mark);
  MakeDecision->Register(&MakeDecision_Mobius_Mark);
}

bool CorpSCADANetworksADVISE::DoNothingOutcome1Step::Enabled() {
  OldEnabled = NewEnabled;
  NewEnabled = (((*(DoNothingChosen_Mobius_Mark)) >= 1));
  return NewEnabled;
}

double CorpSCADANetworksADVISE::DoNothingOutcome1Step::timeDistributionParameter0() {
  return 1;
}

double CorpSCADANetworksADVISE::DoNothingOutcome1Step::Weight() {
  return 1;
}

bool CorpSCADANetworksADVISE::DoNothingOutcome1Step::ReactivationPredicate() {
  return false;
}

bool CorpSCADANetworksADVISE::DoNothingOutcome1Step::ReactivationFunction() {
  return false;
}

double CorpSCADANetworksADVISE::DoNothingOutcome1Step::SampleDistribution() {
  return TheDistribution->Deterministic(
timeDistributionParameter0());
}

double *CorpSCADANetworksADVISE::DoNothingOutcome1Step::ReturnDistributionParameters() {
  TheDistributionParameters[0] = timeDistributionParameter0();
  return TheDistributionParameters;
}

int CorpSCADANetworksADVISE::DoNothingOutcome1Step::Rank() {
  return 1;
}

bool CorpSCADANetworksADVISE::DoNothingOutcome1Step::preconditionsMet() {
  
  return true;
}

void CorpSCADANetworksADVISE::DoNothingOutcome1Step::executeEffects() {
  
}

double CorpSCADANetworksADVISE::DoNothingOutcome1Step::getCost() {
  return 0;
}

double CorpSCADANetworksADVISE::DoNothingOutcome1Step::getOutcomeProbability() {
  return 1;
}

double CorpSCADANetworksADVISE::DoNothingOutcome1Step::getDetection() {
  return 0;
}

/*====================== SCADALAN_InstallMalwareOnNetworkFailureStep ========================*/

CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::SCADALAN_InstallMalwareOnNetworkFailureStep() {
  TheDistributionParameters = new double[1];
  commonInit("SCADALAN_InstallMalwareOnNetworkFailureStep", 1, Exponential, RaceEnabled, 9, 2, false);
  setStepName("SCADALAN_InstallMalwareOnNetwork");
}

CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::~SCADALAN_InstallMalwareOnNetworkFailureStep() {
  delete[] TheDistributionParameters;
}

void CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::LinkVariables() {
  SCADALAN_MalwareInstalled->Register(&SCADALAN_MalwareInstalled_Mobius_Mark);
  MalwareonSCADANet->Register(&MalwareonSCADANet_Mobius_Mark);
  SCADALAN_NetworkAccess->Register(&SCADALAN_NetworkAccess_Mobius_Mark);
  SCADALAN_isIDSOperational->Register(&SCADALAN_isIDSOperational_Mobius_Mark);
  SCADALAN_InstallMalwareOnNetworkChosen->Register(&SCADALAN_InstallMalwareOnNetworkChosen_Mobius_Mark);
  DoNothingWeight->Register(&DoNothingWeight_Mobius_Mark);
  SCADALAN_InstallMalwareOnNetworkWeight->Register(&SCADALAN_InstallMalwareOnNetworkWeight_Mobius_Mark);
  CorpSCADAFW_DefeatFirewallWeight->Register(&CorpSCADAFW_DefeatFirewallWeight_Mobius_Mark);
  MakeDecision->Register(&MakeDecision_Mobius_Mark);
}

bool CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::Enabled() {
  OldEnabled = NewEnabled;
  NewEnabled = (((*(SCADALAN_InstallMalwareOnNetworkChosen_Mobius_Mark)) >= 1));
  return NewEnabled;
}

double CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::timeDistributionParameter0() {
  return 0.5;
}

double CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::Weight() {
return 0.1;
}

bool CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::ReactivationPredicate() {
  return false;
}

bool CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::ReactivationFunction() {
  return false;
}

double CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::SampleDistribution() {
  return TheDistribution->Exponential(
timeDistributionParameter0());
}

double *CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::ReturnDistributionParameters() {
  TheDistributionParameters[0] = timeDistributionParameter0();
  return TheDistributionParameters;
}

int CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::Rank() {
  return 1;
}

bool CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::preconditionsMet() {
  return SCADALAN_NetworkAccess->Mark();
  return true;
}

void CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::executeEffects() {
  // Goal Definition for "Malware on SCADA Net"
auto MalwareonSCADANetDef = [&](){ return SCADALAN_MalwareInstalled->Mark();};
MalwareonSCADANet->Mark() = MalwareonSCADANetDef();
}

double CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::getCost() {
  return 10;
}

double CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::getOutcomeProbability() {
  return 0.1;
}

double CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkFailureStep::getDetection() {
  if (SCADALAN_isIDSOperational->Mark())
	return 0.99;

return 0.2;
}

/*====================== SCADALAN_InstallMalwareOnNetworkSuccessStep ========================*/

CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::SCADALAN_InstallMalwareOnNetworkSuccessStep() {
  TheDistributionParameters = new double[1];
  commonInit("SCADALAN_InstallMalwareOnNetworkSuccessStep", 1, Exponential, RaceEnabled, 9, 2, false);
  setStepName("SCADALAN_InstallMalwareOnNetwork");
}

CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::~SCADALAN_InstallMalwareOnNetworkSuccessStep() {
  delete[] TheDistributionParameters;
}

void CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::LinkVariables() {
  SCADALAN_MalwareInstalled->Register(&SCADALAN_MalwareInstalled_Mobius_Mark);
  MalwareonSCADANet->Register(&MalwareonSCADANet_Mobius_Mark);
  SCADALAN_NetworkAccess->Register(&SCADALAN_NetworkAccess_Mobius_Mark);
  SCADALAN_isIDSOperational->Register(&SCADALAN_isIDSOperational_Mobius_Mark);
  SCADALAN_InstallMalwareOnNetworkChosen->Register(&SCADALAN_InstallMalwareOnNetworkChosen_Mobius_Mark);
  DoNothingWeight->Register(&DoNothingWeight_Mobius_Mark);
  SCADALAN_InstallMalwareOnNetworkWeight->Register(&SCADALAN_InstallMalwareOnNetworkWeight_Mobius_Mark);
  CorpSCADAFW_DefeatFirewallWeight->Register(&CorpSCADAFW_DefeatFirewallWeight_Mobius_Mark);
  MakeDecision->Register(&MakeDecision_Mobius_Mark);
}

bool CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::Enabled() {
  OldEnabled = NewEnabled;
  NewEnabled = (((*(SCADALAN_InstallMalwareOnNetworkChosen_Mobius_Mark)) >= 1));
  return NewEnabled;
}

double CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::timeDistributionParameter0() {
  return 0.5;
}

double CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::Weight() {
return 0.9;
}

bool CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::ReactivationPredicate() {
  return false;
}

bool CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::ReactivationFunction() {
  return false;
}

double CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::SampleDistribution() {
  return TheDistribution->Exponential(
timeDistributionParameter0());
}

double *CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::ReturnDistributionParameters() {
  TheDistributionParameters[0] = timeDistributionParameter0();
  return TheDistributionParameters;
}

int CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::Rank() {
  return 1;
}

bool CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::preconditionsMet() {
  return SCADALAN_NetworkAccess->Mark();
  return true;
}

void CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::executeEffects() {
  SCADALAN_MalwareInstalled->Mark() = 1;

// Goal Definition for "Malware on SCADA Net"
auto MalwareonSCADANetDef = [&](){ return SCADALAN_MalwareInstalled->Mark();};
MalwareonSCADANet->Mark() = MalwareonSCADANetDef();
}

double CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::getCost() {
  return 10;
}

double CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::getOutcomeProbability() {
  return 0.9;
}

double CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkSuccessStep::getDetection() {
  if (SCADALAN_isIDSOperational->Mark())
	return 0.6;

return 0.1;
}

/*====================== CorpSCADAFW_DefeatFirewallFailureStep ========================*/

CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::CorpSCADAFW_DefeatFirewallFailureStep() {
  TheDistributionParameters = new double[2];
  commonInit("CorpSCADAFW_DefeatFirewallFailureStep", 2, Normal, RaceEnabled, 8, 2, false);
  setStepName("CorpSCADAFW_DefeatFirewall");
}

CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::~CorpSCADAFW_DefeatFirewallFailureStep() {
  delete[] TheDistributionParameters;
}

void CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::LinkVariables() {
  CorporateLAN_NetworkAccess->Register(&CorporateLAN_NetworkAccess_Mobius_Mark);
  SCADALAN_NetworkAccess->Register(&SCADALAN_NetworkAccess_Mobius_Mark);
  BruteForceSkill->Register(&BruteForceSkill_Mobius_Mark);
  CorpSCADAFW_DefeatFirewallChosen->Register(&CorpSCADAFW_DefeatFirewallChosen_Mobius_Mark);
  DoNothingWeight->Register(&DoNothingWeight_Mobius_Mark);
  SCADALAN_InstallMalwareOnNetworkWeight->Register(&SCADALAN_InstallMalwareOnNetworkWeight_Mobius_Mark);
  CorpSCADAFW_DefeatFirewallWeight->Register(&CorpSCADAFW_DefeatFirewallWeight_Mobius_Mark);
  MakeDecision->Register(&MakeDecision_Mobius_Mark);
}

bool CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::Enabled() {
  OldEnabled = NewEnabled;
  NewEnabled = (((*(CorpSCADAFW_DefeatFirewallChosen_Mobius_Mark)) >= 1));
  return NewEnabled;
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::timeDistributionParameter0() {
  if (BruteForceSkill->Mark() > 600)
	return 20;

return 40;
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::timeDistributionParameter1() {
  return 10;
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::Weight() {
return 0.7;
}

bool CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::ReactivationPredicate() {
  return false;
}

bool CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::ReactivationFunction() {
  return false;
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::SampleDistribution() {
  return TheDistribution->Normal(
timeDistributionParameter0(), timeDistributionParameter1());
}

double *CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::ReturnDistributionParameters() {
  TheDistributionParameters[0] = timeDistributionParameter0();
  TheDistributionParameters[1] = timeDistributionParameter1();
  return TheDistributionParameters;
}

int CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::Rank() {
  return 1;
}

bool CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::preconditionsMet() {
  std::array<ADVISEStateVariable<short>*, 2> theNetworkAccesses = {CorporateLAN_NetworkAccess, SCADALAN_NetworkAccess};
bool oneNotPossessed = false;
bool atleastOnePossessed = false;

for (const auto& n : theNetworkAccesses) {
	if (n->Mark())
		atleastOnePossessed = true;
	else
		oneNotPossessed = true;
}

return atleastOnePossessed && oneNotPossessed;
  return true;
}

void CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::executeEffects() {
  
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::getCost() {
  return 35;
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::getOutcomeProbability() {
  return 0.7;
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallFailureStep::getDetection() {
  return 0.4;
}

/*====================== CorpSCADAFW_DefeatFirewallSuccessStep ========================*/

CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::CorpSCADAFW_DefeatFirewallSuccessStep() {
  TheDistributionParameters = new double[2];
  commonInit("CorpSCADAFW_DefeatFirewallSuccessStep", 2, Normal, RaceEnabled, 8, 2, false);
  setStepName("CorpSCADAFW_DefeatFirewall");
}

CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::~CorpSCADAFW_DefeatFirewallSuccessStep() {
  delete[] TheDistributionParameters;
}

void CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::LinkVariables() {
  CorporateLAN_NetworkAccess->Register(&CorporateLAN_NetworkAccess_Mobius_Mark);
  SCADALAN_NetworkAccess->Register(&SCADALAN_NetworkAccess_Mobius_Mark);
  BruteForceSkill->Register(&BruteForceSkill_Mobius_Mark);
  CorpSCADAFW_DefeatFirewallChosen->Register(&CorpSCADAFW_DefeatFirewallChosen_Mobius_Mark);
  DoNothingWeight->Register(&DoNothingWeight_Mobius_Mark);
  SCADALAN_InstallMalwareOnNetworkWeight->Register(&SCADALAN_InstallMalwareOnNetworkWeight_Mobius_Mark);
  CorpSCADAFW_DefeatFirewallWeight->Register(&CorpSCADAFW_DefeatFirewallWeight_Mobius_Mark);
  MakeDecision->Register(&MakeDecision_Mobius_Mark);
}

bool CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::Enabled() {
  OldEnabled = NewEnabled;
  NewEnabled = (((*(CorpSCADAFW_DefeatFirewallChosen_Mobius_Mark)) >= 1));
  return NewEnabled;
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::timeDistributionParameter0() {
  if (BruteForceSkill->Mark() > 600)
	return 20;

return 40;
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::timeDistributionParameter1() {
  return 10;
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::Weight() {
return 0.3;
}

bool CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::ReactivationPredicate() {
  return false;
}

bool CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::ReactivationFunction() {
  return false;
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::SampleDistribution() {
  return TheDistribution->Normal(
timeDistributionParameter0(), timeDistributionParameter1());
}

double *CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::ReturnDistributionParameters() {
  TheDistributionParameters[0] = timeDistributionParameter0();
  TheDistributionParameters[1] = timeDistributionParameter1();
  return TheDistributionParameters;
}

int CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::Rank() {
  return 1;
}

bool CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::preconditionsMet() {
  std::array<ADVISEStateVariable<short>*, 2> theNetworkAccesses = {CorporateLAN_NetworkAccess, SCADALAN_NetworkAccess};
bool oneNotPossessed = false;
bool atleastOnePossessed = false;

for (const auto& n : theNetworkAccesses) {
	if (n->Mark())
		atleastOnePossessed = true;
	else
		oneNotPossessed = true;
}

return atleastOnePossessed && oneNotPossessed;
  return true;
}

void CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::executeEffects() {
  std::array<ADVISEStateVariable<short>*, 2> theNetworkAccesses = {CorporateLAN_NetworkAccess, SCADALAN_NetworkAccess};
for (const auto& n : theNetworkAccesses)
	n->Mark() = 1;
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::getCost() {
  return 35;
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::getOutcomeProbability() {
  return 0.3;
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallSuccessStep::getDetection() {
  return 0.05;
}

/*****************************************************************/
/*                   Adversary Decisions                         */
/*****************************************************************/

/*====================== DoNothingAdversaryDecision ========================*/

CorpSCADANetworksADVISE::DoNothingAdversaryDecision::DoNothingAdversaryDecision() {
  commonInit("DoNothingAdversaryDecision", 3, Instantaneous, RaceEnabled, 2, 1, false);
}

CorpSCADANetworksADVISE::DoNothingAdversaryDecision::~DoNothingAdversaryDecision() {
}

void CorpSCADANetworksADVISE::DoNothingAdversaryDecision::LinkVariables() {
  MakeDecision->Register(&MakeDecision_Mobius_Mark);
  DoNothingChosen->Register(&DoNothingChosen_Mobius_Mark);
  DoNothingWeight->Register(&DoNothingWeight_Mobius_Mark);
}

bool CorpSCADANetworksADVISE::DoNothingAdversaryDecision::Enabled() {
  OldEnabled = NewEnabled;
  NewEnabled = (((*(MakeDecision_Mobius_Mark)) >= 1));
  return NewEnabled;
}

double CorpSCADANetworksADVISE::DoNothingAdversaryDecision::Weight() {
  return DoNothingWeight->Mark();
}

bool CorpSCADANetworksADVISE::DoNothingAdversaryDecision::ReactivationPredicate() {
  return false;
}

bool CorpSCADANetworksADVISE::DoNothingAdversaryDecision::ReactivationFunction() {
  return false;
}

double CorpSCADANetworksADVISE::DoNothingAdversaryDecision::SampleDistribution() {
  return 0;
}

double* CorpSCADANetworksADVISE::DoNothingAdversaryDecision::ReturnDistributionParameters() {
  return NULL;
}

int CorpSCADANetworksADVISE::DoNothingAdversaryDecision::Rank() {
  return 1;
}

BaseActionClass* CorpSCADANetworksADVISE::DoNothingAdversaryDecision::Fire() {
  (*(MakeDecision_Mobius_Mark))--;
  (*(DoNothingWeight_Mobius_Mark))--;
  (*(DoNothingChosen_Mobius_Mark))++;
  return this;
}

/*====================== SCADALAN_InstallMalwareOnNetworkAdversaryDecision ========================*/

CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkAdversaryDecision::SCADALAN_InstallMalwareOnNetworkAdversaryDecision() {
  commonInit("SCADALAN_InstallMalwareOnNetworkAdversaryDecision", 3, Instantaneous, RaceEnabled, 2, 1, false);
}

CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkAdversaryDecision::~SCADALAN_InstallMalwareOnNetworkAdversaryDecision() {
}

void CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkAdversaryDecision::LinkVariables() {
  MakeDecision->Register(&MakeDecision_Mobius_Mark);
  SCADALAN_InstallMalwareOnNetworkChosen->Register(&SCADALAN_InstallMalwareOnNetworkChosen_Mobius_Mark);
  SCADALAN_InstallMalwareOnNetworkWeight->Register(&SCADALAN_InstallMalwareOnNetworkWeight_Mobius_Mark);
}

bool CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkAdversaryDecision::Enabled() {
  OldEnabled = NewEnabled;
  NewEnabled = (((*(MakeDecision_Mobius_Mark)) >= 1));
  return NewEnabled;
}

double CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkAdversaryDecision::Weight() {
  return SCADALAN_InstallMalwareOnNetworkWeight->Mark();
}

bool CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkAdversaryDecision::ReactivationPredicate() {
  return false;
}

bool CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkAdversaryDecision::ReactivationFunction() {
  return false;
}

double CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkAdversaryDecision::SampleDistribution() {
  return 0;
}

double* CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkAdversaryDecision::ReturnDistributionParameters() {
  return NULL;
}

int CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkAdversaryDecision::Rank() {
  return 1;
}

BaseActionClass* CorpSCADANetworksADVISE::SCADALAN_InstallMalwareOnNetworkAdversaryDecision::Fire() {
  (*(MakeDecision_Mobius_Mark))--;
  (*(SCADALAN_InstallMalwareOnNetworkWeight_Mobius_Mark))--;
  (*(SCADALAN_InstallMalwareOnNetworkChosen_Mobius_Mark))++;
  return this;
}

/*====================== CorpSCADAFW_DefeatFirewallAdversaryDecision ========================*/

CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallAdversaryDecision::CorpSCADAFW_DefeatFirewallAdversaryDecision() {
  commonInit("CorpSCADAFW_DefeatFirewallAdversaryDecision", 3, Instantaneous, RaceEnabled, 2, 1, false);
}

CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallAdversaryDecision::~CorpSCADAFW_DefeatFirewallAdversaryDecision() {
}

void CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallAdversaryDecision::LinkVariables() {
  MakeDecision->Register(&MakeDecision_Mobius_Mark);
  CorpSCADAFW_DefeatFirewallChosen->Register(&CorpSCADAFW_DefeatFirewallChosen_Mobius_Mark);
  CorpSCADAFW_DefeatFirewallWeight->Register(&CorpSCADAFW_DefeatFirewallWeight_Mobius_Mark);
}

bool CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallAdversaryDecision::Enabled() {
  OldEnabled = NewEnabled;
  NewEnabled = (((*(MakeDecision_Mobius_Mark)) >= 1));
  return NewEnabled;
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallAdversaryDecision::Weight() {
  return CorpSCADAFW_DefeatFirewallWeight->Mark();
}

bool CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallAdversaryDecision::ReactivationPredicate() {
  return false;
}

bool CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallAdversaryDecision::ReactivationFunction() {
  return false;
}

double CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallAdversaryDecision::SampleDistribution() {
  return 0;
}

double* CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallAdversaryDecision::ReturnDistributionParameters() {
  return NULL;
}

int CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallAdversaryDecision::Rank() {
  return 1;
}

BaseActionClass* CorpSCADANetworksADVISE::CorpSCADAFW_DefeatFirewallAdversaryDecision::Fire() {
  (*(MakeDecision_Mobius_Mark))--;
  (*(CorpSCADAFW_DefeatFirewallWeight_Mobius_Mark))--;
  (*(CorpSCADAFW_DefeatFirewallChosen_Mobius_Mark))++;
  return this;
}

