#ifndef CorpSCADANetworksADVISE_H_
#define CorpSCADANetworksADVISE_H_

#include "Simulator/UserDistributions.h"
#include "BaseClasses/GlobalVariables.h"
#include "BaseClasses/PostselectGroup.h"
#include "BaseClasses/PreselectGroup.h"
#include "BaseClasses/atomic/advise/ADVISEModel.h"
#include "BaseClasses/atomic/advise/Access.h"
#include "BaseClasses/atomic/advise/AdversaryDecision.h"
#include "BaseClasses/atomic/advise/BeginAdversaryDecision.h"
#include "BaseClasses/atomic/advise/Goal.h"
#include "BaseClasses/atomic/advise/Knowledge.h"
#include "BaseClasses/atomic/advise/Skill.h"
#include "BaseClasses/atomic/advise/Step.h"
#include "BaseClasses/atomic/advise/StepChosen.h"
#include "BaseClasses/atomic/advise/StepWeight.h"
#include "BaseClasses/atomic/advise/SystemSV.h"
#include <limits.h>
#include <math.h>
extern UserDistributions* TheDistribution;

/*********************************************************************
               CorpSCADANetworksADVISE Submodel Definition                   
*********************************************************************/

class CorpSCADANetworksADVISE : public ADVISEModel {
public:

class DoNothingAdversaryDecision : public AdversaryDecision {
public:

  BeginAdversaryDecision* MakeDecision;
  short* MakeDecision_Mobius_Mark;
  StepChosen *DoNothingChosen;
  short* DoNothingChosen_Mobius_Mark;
  StepWeight *DoNothingWeight;
  short* DoNothingWeight_Mobius_Mark;
  double* TheDistributionParameters;

  DoNothingAdversaryDecision();
  ~DoNothingAdversaryDecision();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass *Fire();
}; // DoNothingAdversaryDecision

class SCADALAN_InstallMalwareOnNetworkAdversaryDecision : public AdversaryDecision {
public:

  BeginAdversaryDecision* MakeDecision;
  short* MakeDecision_Mobius_Mark;
  StepChosen *SCADALAN_InstallMalwareOnNetworkChosen;
  short* SCADALAN_InstallMalwareOnNetworkChosen_Mobius_Mark;
  StepWeight *SCADALAN_InstallMalwareOnNetworkWeight;
  short* SCADALAN_InstallMalwareOnNetworkWeight_Mobius_Mark;
  double* TheDistributionParameters;

  SCADALAN_InstallMalwareOnNetworkAdversaryDecision();
  ~SCADALAN_InstallMalwareOnNetworkAdversaryDecision();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass *Fire();
}; // SCADALAN_InstallMalwareOnNetworkAdversaryDecision

class CorpSCADAFW_DefeatFirewallAdversaryDecision : public AdversaryDecision {
public:

  BeginAdversaryDecision* MakeDecision;
  short* MakeDecision_Mobius_Mark;
  StepChosen *CorpSCADAFW_DefeatFirewallChosen;
  short* CorpSCADAFW_DefeatFirewallChosen_Mobius_Mark;
  StepWeight *CorpSCADAFW_DefeatFirewallWeight;
  short* CorpSCADAFW_DefeatFirewallWeight_Mobius_Mark;
  double* TheDistributionParameters;

  CorpSCADAFW_DefeatFirewallAdversaryDecision();
  ~CorpSCADAFW_DefeatFirewallAdversaryDecision();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass *Fire();
}; // CorpSCADAFW_DefeatFirewallAdversaryDecision

class DoNothingOutcome1Step : public Step {
public:

  StepChosen* DoNothingChosen;
  short* DoNothingChosen_Mobius_Mark;
  StepWeight* DoNothingWeight;
  short* DoNothingWeight_Mobius_Mark;
  StepWeight* SCADALAN_InstallMalwareOnNetworkWeight;
  short* SCADALAN_InstallMalwareOnNetworkWeight_Mobius_Mark;
  StepWeight* CorpSCADAFW_DefeatFirewallWeight;
  short* CorpSCADAFW_DefeatFirewallWeight_Mobius_Mark;
  BeginAdversaryDecision *MakeDecision;
  short* MakeDecision_Mobius_Mark;
  double* TheDistributionParameters;

  DoNothingOutcome1Step();
  ~DoNothingOutcome1Step();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  double getCost();
  double getOutcomeProbability();
  double getDetection();
  int Rank();
  double timeDistributionParameter0();
  bool preconditionsMet();
  void executeEffects();
}; // DoNothingOutcome1Step

class SCADALAN_InstallMalwareOnNetworkFailureStep : public Step {
public:

  SystemSV<bool>* SCADALAN_MalwareInstalled;
  bool* SCADALAN_MalwareInstalled_Mobius_Mark;
  Goal* MalwareonSCADANet;
  short* MalwareonSCADANet_Mobius_Mark;
  Access* SCADALAN_NetworkAccess;
  short* SCADALAN_NetworkAccess_Mobius_Mark;
  SystemSV<bool>* SCADALAN_isIDSOperational;
  bool* SCADALAN_isIDSOperational_Mobius_Mark;
  StepChosen* SCADALAN_InstallMalwareOnNetworkChosen;
  short* SCADALAN_InstallMalwareOnNetworkChosen_Mobius_Mark;
  StepWeight* DoNothingWeight;
  short* DoNothingWeight_Mobius_Mark;
  StepWeight* SCADALAN_InstallMalwareOnNetworkWeight;
  short* SCADALAN_InstallMalwareOnNetworkWeight_Mobius_Mark;
  StepWeight* CorpSCADAFW_DefeatFirewallWeight;
  short* CorpSCADAFW_DefeatFirewallWeight_Mobius_Mark;
  BeginAdversaryDecision *MakeDecision;
  short* MakeDecision_Mobius_Mark;
  double* TheDistributionParameters;

  SCADALAN_InstallMalwareOnNetworkFailureStep();
  ~SCADALAN_InstallMalwareOnNetworkFailureStep();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  double getCost();
  double getOutcomeProbability();
  double getDetection();
  int Rank();
  double timeDistributionParameter0();
  bool preconditionsMet();
  void executeEffects();
}; // SCADALAN_InstallMalwareOnNetworkFailureStep

class SCADALAN_InstallMalwareOnNetworkSuccessStep : public Step {
public:

  SystemSV<bool>* SCADALAN_MalwareInstalled;
  bool* SCADALAN_MalwareInstalled_Mobius_Mark;
  Goal* MalwareonSCADANet;
  short* MalwareonSCADANet_Mobius_Mark;
  Access* SCADALAN_NetworkAccess;
  short* SCADALAN_NetworkAccess_Mobius_Mark;
  SystemSV<bool>* SCADALAN_isIDSOperational;
  bool* SCADALAN_isIDSOperational_Mobius_Mark;
  StepChosen* SCADALAN_InstallMalwareOnNetworkChosen;
  short* SCADALAN_InstallMalwareOnNetworkChosen_Mobius_Mark;
  StepWeight* DoNothingWeight;
  short* DoNothingWeight_Mobius_Mark;
  StepWeight* SCADALAN_InstallMalwareOnNetworkWeight;
  short* SCADALAN_InstallMalwareOnNetworkWeight_Mobius_Mark;
  StepWeight* CorpSCADAFW_DefeatFirewallWeight;
  short* CorpSCADAFW_DefeatFirewallWeight_Mobius_Mark;
  BeginAdversaryDecision *MakeDecision;
  short* MakeDecision_Mobius_Mark;
  double* TheDistributionParameters;

  SCADALAN_InstallMalwareOnNetworkSuccessStep();
  ~SCADALAN_InstallMalwareOnNetworkSuccessStep();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  double getCost();
  double getOutcomeProbability();
  double getDetection();
  int Rank();
  double timeDistributionParameter0();
  bool preconditionsMet();
  void executeEffects();
}; // SCADALAN_InstallMalwareOnNetworkSuccessStep

class CorpSCADAFW_DefeatFirewallFailureStep : public Step {
public:

  Access* CorporateLAN_NetworkAccess;
  short* CorporateLAN_NetworkAccess_Mobius_Mark;
  Access* SCADALAN_NetworkAccess;
  short* SCADALAN_NetworkAccess_Mobius_Mark;
  Skill* BruteForceSkill;
  short* BruteForceSkill_Mobius_Mark;
  StepChosen* CorpSCADAFW_DefeatFirewallChosen;
  short* CorpSCADAFW_DefeatFirewallChosen_Mobius_Mark;
  StepWeight* DoNothingWeight;
  short* DoNothingWeight_Mobius_Mark;
  StepWeight* SCADALAN_InstallMalwareOnNetworkWeight;
  short* SCADALAN_InstallMalwareOnNetworkWeight_Mobius_Mark;
  StepWeight* CorpSCADAFW_DefeatFirewallWeight;
  short* CorpSCADAFW_DefeatFirewallWeight_Mobius_Mark;
  BeginAdversaryDecision *MakeDecision;
  short* MakeDecision_Mobius_Mark;
  double* TheDistributionParameters;

  CorpSCADAFW_DefeatFirewallFailureStep();
  ~CorpSCADAFW_DefeatFirewallFailureStep();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  double getCost();
  double getOutcomeProbability();
  double getDetection();
  int Rank();
  double timeDistributionParameter0();
  double timeDistributionParameter1();
  bool preconditionsMet();
  void executeEffects();
}; // CorpSCADAFW_DefeatFirewallFailureStep

class CorpSCADAFW_DefeatFirewallSuccessStep : public Step {
public:

  Access* CorporateLAN_NetworkAccess;
  short* CorporateLAN_NetworkAccess_Mobius_Mark;
  Access* SCADALAN_NetworkAccess;
  short* SCADALAN_NetworkAccess_Mobius_Mark;
  Skill* BruteForceSkill;
  short* BruteForceSkill_Mobius_Mark;
  StepChosen* CorpSCADAFW_DefeatFirewallChosen;
  short* CorpSCADAFW_DefeatFirewallChosen_Mobius_Mark;
  StepWeight* DoNothingWeight;
  short* DoNothingWeight_Mobius_Mark;
  StepWeight* SCADALAN_InstallMalwareOnNetworkWeight;
  short* SCADALAN_InstallMalwareOnNetworkWeight_Mobius_Mark;
  StepWeight* CorpSCADAFW_DefeatFirewallWeight;
  short* CorpSCADAFW_DefeatFirewallWeight_Mobius_Mark;
  BeginAdversaryDecision *MakeDecision;
  short* MakeDecision_Mobius_Mark;
  double* TheDistributionParameters;

  CorpSCADAFW_DefeatFirewallSuccessStep();
  ~CorpSCADAFW_DefeatFirewallSuccessStep();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  double getCost();
  double getOutcomeProbability();
  double getDetection();
  int Rank();
  double timeDistributionParameter0();
  double timeDistributionParameter1();
  bool preconditionsMet();
  void executeEffects();
}; // CorpSCADAFW_DefeatFirewallSuccessStep

  //List of user-specified state variables
  Access *SCADALAN_NetworkAccess;
  Access *CorporateLAN_NetworkAccess;
  Skill *BruteForceSkill;
  Goal *MalwareonSCADANet;
  SystemSV<bool> *SCADALAN_MalwareInstalled;
  SystemSV<bool> *SCADALAN_isIDSOperational;
  //List of attack step weight state variables
  StepWeight *DoNothingWeight;
  StepWeight *SCADALAN_InstallMalwareOnNetworkWeight;
  StepWeight *CorpSCADAFW_DefeatFirewallWeight;
  //List of attack step chosen state variables
  StepChosen *DoNothingChosen;
  StepChosen *SCADALAN_InstallMalwareOnNetworkChosen;
  StepChosen *CorpSCADAFW_DefeatFirewallChosen;
  //List of attack steps
  DoNothingOutcome1Step DoNothingOutcome1;
  SCADALAN_InstallMalwareOnNetworkFailureStep SCADALAN_InstallMalwareOnNetworkFailure;
  SCADALAN_InstallMalwareOnNetworkSuccessStep SCADALAN_InstallMalwareOnNetworkSuccess;
  CorpSCADAFW_DefeatFirewallFailureStep CorpSCADAFW_DefeatFirewallFailure;
  CorpSCADAFW_DefeatFirewallSuccessStep CorpSCADAFW_DefeatFirewallSuccess;
  DoNothingAdversaryDecision DoNothingAD;
  SCADALAN_InstallMalwareOnNetworkAdversaryDecision SCADALAN_InstallMalwareOnNetworkAD;
  CorpSCADAFW_DefeatFirewallAdversaryDecision CorpSCADAFW_DefeatFirewallAD;
  //Groups for attack steps
  PreselectGroup ImmediateGroup;
  PostselectGroup SCADALAN_InstallMalwareOnNetworkGroup;
  PostselectGroup CorpSCADAFW_DefeatFirewallGroup;
  PostselectGroup AdversaryDecisionGroup;
  CorpSCADANetworksADVISE();
  ~CorpSCADANetworksADVISE();
  void assignSVsToAttackSteps();
  double convertCostToUtility(double cost);
  double convertDetectionToUtility(double detections);
  double convertPayoffToUtility(double payoff);
  void customInitialization();

protected:
}; // end CorpSCADANetworksADVISE

#endif // CorpSCADANetworksADVISE_H_
